import Chai = require("chai");
import MiddlewareService from "../src/services/middleware";
import {inject} from '../src/testing/inject';
import SendResponseMiddleware from "../src/middlewares/send-response";
import {FakeResponse} from "./helper/FakeResponse";
let expect: Chai.ExpectStatic = Chai.expect;

describe('SendResponseMiddleware :', () => {


    it('should send response (boolean)', inject([MiddlewareService], (middlewareService: MiddlewareService) => {
        const fakeResponse = new FakeResponse();
        const middleware = middlewareService.invoke<SendResponseMiddleware>(SendResponseMiddleware);

        expect(middleware).not.to.be.undefined;

        middleware.use(true, fakeResponse as any);

        expect(fakeResponse._body).to.be.a('string');
        expect(fakeResponse._body).to.equal('true');
    }));

    it('should send response (number)', inject([MiddlewareService], (middlewareService: MiddlewareService) => {
        const fakeResponse = new FakeResponse();
        const middleware = middlewareService.invoke<SendResponseMiddleware>(SendResponseMiddleware);

        expect(middleware).not.to.be.undefined;

        middleware.use(1, fakeResponse as any);

        expect(fakeResponse._body).to.be.a('string');
        expect(fakeResponse._body).to.equal('1');
    }));

    it('should send response (null)', inject([MiddlewareService], (middlewareService: MiddlewareService) => {
        const fakeResponse = new FakeResponse();
        const middleware = middlewareService.invoke<SendResponseMiddleware>(SendResponseMiddleware);

        expect(middleware).not.to.be.undefined;

        middleware.use(null, fakeResponse as any);

        expect(fakeResponse._body).to.be.a('string');
        expect(fakeResponse._body).to.equal('null');
    }));

    it('should send response (date)', inject([MiddlewareService], (middlewareService: MiddlewareService) => {
        const fakeResponse = new FakeResponse();
        const date = new Date();
        const middleware = middlewareService.invoke<SendResponseMiddleware>(SendResponseMiddleware);

        expect(middleware).not.to.be.undefined;

        middleware.use(date, fakeResponse as any);

        expect(fakeResponse._body).to.be.a('string');
        expect(fakeResponse._headers).to.contains('Content-Type:text/json');
        expect(fakeResponse._body).to.equal(JSON.stringify(date));

    }));

    it('should send nothing (undefined)', inject([MiddlewareService], (middlewareService: MiddlewareService) => {
        const fakeResponse = new FakeResponse();
        const middleware = middlewareService.invoke<SendResponseMiddleware>(SendResponseMiddleware);

        expect(middleware).not.to.be.undefined;

        middleware.use(undefined, fakeResponse as any);

        expect(fakeResponse._body).to.equal('');

    }));

    it('should send nothing (headersSent)', inject([MiddlewareService], (middlewareService: MiddlewareService) => {
        const fakeResponse = new FakeResponse();
        const middleware = middlewareService.invoke<SendResponseMiddleware>(SendResponseMiddleware);
        fakeResponse['headersSent'] = true;

        expect(middleware).not.to.be.undefined;

        middleware.use(true, fakeResponse as any);

        expect(fakeResponse._body).to.equal('');

    }));

    it('should send response (object)', inject([MiddlewareService], (middlewareService: MiddlewareService) => {
        const fakeResponse = new FakeResponse();
        const middleware = middlewareService.invoke<SendResponseMiddleware>(SendResponseMiddleware);
        const obj = {};

        expect(middleware).not.to.be.undefined;

        middleware.use(obj, fakeResponse as any);

        expect(fakeResponse._body).to.be.a('string');
        expect(fakeResponse._headers).to.contains('Content-Type:text/json');
        expect(fakeResponse._body).to.equal(JSON.stringify(obj));

    }));

});